#A* -------------------------------------------------------------------
#B* This file contains source code for the PyMOL computer program
#C* copyright 1998-2000 by Warren Lyford Delano of DeLano Scientific. 
#D* -------------------------------------------------------------------
#E* It is unlawful to modify or remove this copyright notice.
#F* -------------------------------------------------------------------
#G* Please see the accompanying LICENSE file for further information. 
#H* -------------------------------------------------------------------
#I* Additional authors of this source file include:
#-* 
#-* 
#-*
#Z* -------------------------------------------------------------------

# pmgui_tk 
# TKinter based gui for PyMol
# NOTE: must have treads support compiled into python to use this module
#
# **This is the only module which should be/need be imported by 
# **PyMol Programs

import threading
import sys
from wxPython.wx import *

ID_ABOUT = 101
ID_EXIT  = 102

class MyFrame(wxFrame):
   def __init__(self, parent, ID, title):
      wxFrame.__init__(self, parent, ID, title,
                       wxDefaultPosition, wxSize(200, 150))
      self.CreateStatusBar()
      self.SetStatusText("This is the statusbar")

      menu = wxMenu()
      menu.Append(ID_ABOUT, "&About",
                  "More information about this program")
      menu.AppendSeparator()
      menu.Append(ID_EXIT, "E&xit", "Terminate the program")

      menuBar = wxMenuBar()
      menuBar.Append(menu, "&File");

      self.SetMenuBar(menuBar)


class MyApp(wxApp):
   def OnInit(self):
      frame = MyFrame(NULL, -1, "Hello from wxPython")
      frame.Show(true)
      self.SetTopWindow(frame)
      return true

def run():
   app = MyApp(0)
   app.MainLoop()

t = threading.Thread(target=run,args=())
t.setDaemon(1)
t.start()





