# add your pmg_tk startup scripts to this directory...
# here is an example: (copy, remove comment marks, and save as sample.py)

# BEGIN EXAMPLE: sample.py

#from Tkinter import *
#from pymol import cmd

#def __init__(self):
#   self.menuBar.addmenu('Sample', 'Sample Menu')

#   self.menuBar.addmenuitem('Sample', 'command',
#                      'White background',
#                      label='White Background',
#                     command = lambda : cmd.set("bg_rgb","[1,1,1]"))

#   self.menuBar.addmenuitem('Sample', 'command',
#                      'Black background',
#                      label='Black Background',
#                     command = lambda : cmd.set("bg_rgb","[0,0,0]"))

