# Generated by h2py from glut.h
GLUT_API_VERSION = 3
GLUT_XLIB_IMPLEMENTATION = 12
GLUT_RGB = 0
GLUT_RGBA = GLUT_RGB
GLUT_INDEX = 1
GLUT_SINGLE = 0
GLUT_DOUBLE = 2
GLUT_ACCUM = 4
GLUT_ALPHA = 8
GLUT_DEPTH = 16
GLUT_STENCIL = 32
GLUT_MULTISAMPLE = 128
GLUT_STEREO = 256
GLUT_LUMINANCE = 512
GLUT_LEFT_BUTTON = 0
GLUT_MIDDLE_BUTTON = 1
GLUT_RIGHT_BUTTON = 2
GLUT_DOWN = 0
GLUT_UP = 1
GLUT_KEY_F1 = 1
GLUT_KEY_F2 = 2
GLUT_KEY_F3 = 3
GLUT_KEY_F4 = 4
GLUT_KEY_F5 = 5
GLUT_KEY_F6 = 6
GLUT_KEY_F7 = 7
GLUT_KEY_F8 = 8
GLUT_KEY_F9 = 9
GLUT_KEY_F10 = 10
GLUT_KEY_F11 = 11
GLUT_KEY_F12 = 12
GLUT_KEY_LEFT = 100
GLUT_KEY_UP = 101
GLUT_KEY_RIGHT = 102
GLUT_KEY_DOWN = 103
GLUT_KEY_PAGE_UP = 104
GLUT_KEY_PAGE_DOWN = 105
GLUT_KEY_HOME = 106
GLUT_KEY_END = 107
GLUT_KEY_INSERT = 108
GLUT_LEFT = 0
GLUT_ENTERED = 1
GLUT_MENU_NOT_IN_USE = 0
GLUT_MENU_IN_USE = 1
GLUT_NOT_VISIBLE = 0
GLUT_VISIBLE = 1
GLUT_HIDDEN = 0
GLUT_FULLY_RETAINED = 1
GLUT_PARTIALLY_RETAINED = 2
GLUT_FULLY_COVERED = 3
GLUT_RED = 0
GLUT_GREEN = 1
GLUT_BLUE = 2
GLUT_NORMAL = 0
GLUT_OVERLAY = 1
GLUT_WINDOW_X = 100
GLUT_WINDOW_Y = 101
GLUT_WINDOW_WIDTH = 102
GLUT_WINDOW_HEIGHT = 103
GLUT_WINDOW_BUFFER_SIZE = 104
GLUT_WINDOW_STENCIL_SIZE = 105
GLUT_WINDOW_DEPTH_SIZE = 106
GLUT_WINDOW_RED_SIZE = 107
GLUT_WINDOW_GREEN_SIZE = 108
GLUT_WINDOW_BLUE_SIZE = 109
GLUT_WINDOW_ALPHA_SIZE = 110
GLUT_WINDOW_ACCUM_RED_SIZE = 111
GLUT_WINDOW_ACCUM_GREEN_SIZE = 112
GLUT_WINDOW_ACCUM_BLUE_SIZE = 113
GLUT_WINDOW_ACCUM_ALPHA_SIZE = 114
GLUT_WINDOW_DOUBLEBUFFER = 115
GLUT_WINDOW_RGBA = 116
GLUT_WINDOW_PARENT = 117
GLUT_WINDOW_NUM_CHILDREN = 118
GLUT_WINDOW_COLORMAP_SIZE = 119
GLUT_WINDOW_NUM_SAMPLES = 120
GLUT_WINDOW_STEREO = 121
GLUT_WINDOW_CURSOR = 122
GLUT_SCREEN_WIDTH = 200
GLUT_SCREEN_HEIGHT = 201
GLUT_SCREEN_WIDTH_MM = 202
GLUT_SCREEN_HEIGHT_MM = 203
GLUT_MENU_NUM_ITEMS = 300
GLUT_DISPLAY_MODE_POSSIBLE = 400
GLUT_INIT_WINDOW_X = 500
GLUT_INIT_WINDOW_Y = 501
GLUT_INIT_WINDOW_WIDTH = 502
GLUT_INIT_WINDOW_HEIGHT = 503
GLUT_INIT_DISPLAY_MODE = 504
GLUT_ELAPSED_TIME = 700
GLUT_HAS_KEYBOARD = 600
GLUT_HAS_MOUSE = 601
GLUT_HAS_SPACEBALL = 602
GLUT_HAS_DIAL_AND_BUTTON_BOX = 603
GLUT_HAS_TABLET = 604
GLUT_NUM_MOUSE_BUTTONS = 605
GLUT_NUM_SPACEBALL_BUTTONS = 606
GLUT_NUM_BUTTON_BOX_BUTTONS = 607
GLUT_NUM_DIALS = 608
GLUT_NUM_TABLET_BUTTONS = 609
GLUT_OVERLAY_POSSIBLE = 800
GLUT_LAYER_IN_USE = 801
GLUT_HAS_OVERLAY = 802
GLUT_TRANSPARENT_INDEX = 803
GLUT_NORMAL_DAMAGED = 804
GLUT_OVERLAY_DAMAGED = 805
GLUT_VIDEO_RESIZE_POSSIBLE = 900
GLUT_VIDEO_RESIZE_IN_USE = 901
GLUT_VIDEO_RESIZE_X_DELTA = 902
GLUT_VIDEO_RESIZE_Y_DELTA = 903
GLUT_VIDEO_RESIZE_WIDTH_DELTA = 904
GLUT_VIDEO_RESIZE_HEIGHT_DELTA = 905
GLUT_VIDEO_RESIZE_X = 906
GLUT_VIDEO_RESIZE_Y = 907
GLUT_VIDEO_RESIZE_WIDTH = 908
GLUT_VIDEO_RESIZE_HEIGHT = 909
GLUT_ACTIVE_SHIFT = 1
GLUT_ACTIVE_CTRL = 2
GLUT_ACTIVE_ALT = 4
GLUT_CURSOR_RIGHT_ARROW = 0
GLUT_CURSOR_LEFT_ARROW = 1
GLUT_CURSOR_INFO = 2
GLUT_CURSOR_DESTROY = 3
GLUT_CURSOR_HELP = 4
GLUT_CURSOR_CYCLE = 5
GLUT_CURSOR_SPRAY = 6
GLUT_CURSOR_WAIT = 7
GLUT_CURSOR_TEXT = 8
GLUT_CURSOR_CROSSHAIR = 9
GLUT_CURSOR_UP_DOWN = 10
GLUT_CURSOR_LEFT_RIGHT = 11
GLUT_CURSOR_TOP_SIDE = 12
GLUT_CURSOR_BOTTOM_SIDE = 13
GLUT_CURSOR_LEFT_SIDE = 14
GLUT_CURSOR_RIGHT_SIDE = 15
GLUT_CURSOR_TOP_LEFT_CORNER = 16
GLUT_CURSOR_TOP_RIGHT_CORNER = 17
GLUT_CURSOR_BOTTOM_RIGHT_CORNER = 18
GLUT_CURSOR_BOTTOM_LEFT_CORNER = 19
GLUT_CURSOR_INHERIT = 100
GLUT_CURSOR_NONE = 101
GLUT_CURSOR_FULL_CROSSHAIR = 102
